@@ -275,6 +275,14 @@ class CLIClushTest_A(unittest.TestCase):
         self._clush_t(["-w", HOSTNAME, "/bin/false"], None, b"", 0, exp_err)
         self._clush_t(["-w", HOSTNAME, "-b", "/bin/false"], None, b"", 0, exp_err)
         self._clush_t(["-S", "-w", HOSTNAME, "/bin/false"], None, b"", 1, exp_err)
+        self._clush_t(["--maxrc", "-w", HOSTNAME, "/bin/false"], None, b"", 1, exp_err)
+        self._clush_t(["-O", "maxrc=yes", "-w", HOSTNAME, "/bin/false"], None,
+                      b"", 1, exp_err)
+        # -O takes precedence over --maxrc
+        self._clush_t(["--maxrc", "-O", "maxrc=no", "-w", HOSTNAME, "/bin/false"], None,
+                      b"", 0, exp_err)
+        self._clush_t(["-O", "maxrc=no", "--maxrc", "-w", HOSTNAME, "/bin/false"], None,
+                      b"", 0, exp_err)
         for i in (1, 2, 127, 128, 255):
             s = "clush: %s: exited with exit code %d\n" % (HOSTNAME, i)
             self._clush_t(["-S", "-w", HOSTNAME, "exit %d" % i], None, b"", i,
@@ -606,7 +614,7 @@ class CLIClushTest_B_StdinFailure(unittest.TestCase):
         class BrokenStdinMock(object):
             def isatty(self):
                 return False
-            def read(self, bufsize=1024):
+            def fileno(self):
                 raise IOError(errno.EINVAL, "Invalid argument")
 
         sys.stdin = BrokenStdinMock()
@@ -670,3 +678,50 @@ class CLIClushTest_C_GroupsConf(unittest.TestCase):
         self._clush_t(["--groupsconf", self.custf.name, "-R", "exec", "-w",
                        "@foo", "-bL", "echo ok"], None,
                       b"custom[7-42]: ok\n", 0, b"")
+
+
+class CLIClushTest_D_StdinFIFO(unittest.TestCase):
+    """Unit test class for testing CLI/Clush.py when stdin is a named pipe"""
+
+    def setUp(self):
+        # Create fifo
+        self.fname = tempfile.mktemp()
+        os.mkfifo(self.fname)
+
+        # Launch write thread
+        class FIFOThread(threading.Thread):
+            def run(self):
+                fifo_wr = open(self.fname, "w")
+                fifo_wr.write("0123456789")
+                fifo_wr.close()
+
+        fifoth = FIFOThread()
+        fifoth.fname = self.fname
+        fifoth.start()
+
+        # Use read end of fifo as stdin
+        sys.stdin = open(self.fname, "r")
+
+    def tearDown(self):
+        """cleanup all tasks"""
+        sys.stdin.close()
+        os.unlink(self.fname)
+        task_cleanup()
+        sys.stdin = sys.__stdin__
+
+    def _clush_t(self, args, stdin, expected_stdout, expected_rc=0,
+                 expected_stderr=None):
+        CLI_main(self, main, ['clush'] + args, stdin, expected_stdout,
+                 expected_rc, expected_stderr)
+
+    def test_300_fifo_stdin(self):
+        """test clush with stdin is a fifo (read)"""
+        s = "%s: 0123456789\n" % HOSTNAME
+        self._clush_t(["-w", HOSTNAME, "-v", "cat"], None,
+                      s.encode(), 0, b"")
+
+    def test_301_fifo_stdin(self):
+        """test clush with stdin is a fifo (not read)"""
+        s = "%s: ok\n" % HOSTNAME
+        self._clush_t(["-w", HOSTNAME, "-v", "echo ok"], None,
+                      s.encode(), 0, b"")
