@@ -35,9 +35,10 @@ class CLIClushConfigTest(unittest.TestCase):
         parser.install_connector_options()
         options, _ = parser.parse_args([])
         config = ClushConfig(options, filename=f.name)
-        self.assertEqual(config.color, WHENCOLOR_CHOICES[-1])
+        self.assertEqual(config.color, THREE_CHOICES[0])
         self.assertEqual(config.verbosity, VERB_STD)
         self.assertEqual(config.fanout, 64)
+        self.assertEqual(config.maxrc, False)
         self.assertEqual(config.node_count, True)
         self.assertEqual(config.connect_timeout, 10)
         self.assertEqual(config.command_timeout, 0)
@@ -58,8 +59,9 @@ class CLIClushConfigTest(unittest.TestCase):
         parser.install_connector_options()
         options, _ = parser.parse_args([])
         config = ClushConfig(options, filename=f.name)
-        self.assertEqual(config.color, WHENCOLOR_CHOICES[-1])
+        self.assertEqual(config.color, THREE_CHOICES[0])
         self.assertEqual(config.verbosity, VERB_STD)
+        self.assertEqual(config.maxrc, False)
         self.assertEqual(config.node_count, True)
         self.assertEqual(config.fanout, 64)
         self.assertEqual(config.connect_timeout, 10)
@@ -94,8 +96,9 @@ class CLIClushConfigTest(unittest.TestCase):
         display = Display(options, config)
         display.vprint(VERB_STD, "test")
         display.vprint(VERB_DEBUG, "shouldn't see this")
-        self.assertEqual(config.color, WHENCOLOR_CHOICES[2])
+        self.assertEqual(config.color, THREE_CHOICES[-1])
         self.assertEqual(config.verbosity, VERB_STD)
+        self.assertEqual(config.maxrc, False)
         self.assertEqual(config.node_count, True)
         self.assertEqual(config.fanout, 42)
         self.assertEqual(config.connect_timeout, 14)
@@ -116,6 +119,7 @@ class CLIClushConfigTest(unittest.TestCase):
             command_timeout: 0
             history_size: 100
             color: auto
+            maxrc: yes
             node_count: yes
             verbosity: 1
             ssh_user: root
@@ -130,8 +134,9 @@ class CLIClushConfigTest(unittest.TestCase):
         parser.install_connector_options()
         options, _ = parser.parse_args([])
         config = ClushConfig(options, filename=f.name)
-        self.assertEqual(config.color, WHENCOLOR_CHOICES[2])
+        self.assertEqual(config.color, THREE_CHOICES[-1])
         self.assertEqual(config.verbosity, VERB_STD)
+        self.assertEqual(config.maxrc, True)
         self.assertEqual(config.node_count, True)
         self.assertEqual(config.fanout, 42)
         self.assertEqual(config.connect_timeout, 14)
@@ -293,7 +298,7 @@ class CLIClushConfigTest(unittest.TestCase):
         display = Display(options, config)
         display.vprint(VERB_STD, "test")
         display.vprint(VERB_DEBUG, "test")
-        self.assertEqual(config.color, WHENCOLOR_CHOICES[1])
+        self.assertEqual(config.color, THREE_CHOICES[2])
         self.assertEqual(config.verbosity, VERB_DEBUG) # takes biggest
         self.assertEqual(config.fanout, 36)
         self.assertEqual(config.connect_timeout, 7)
@@ -355,7 +360,7 @@ class CLIClushConfigTest(unittest.TestCase):
             parser.install_connector_options()
             options, _ = parser.parse_args([])
             config = ClushConfig(options) # filename=None to use defaults!
-            self.assertEqual(config.color, WHENCOLOR_CHOICES[0])
+            self.assertEqual(config.color, THREE_CHOICES[1])
             self.assertEqual(config.verbosity, VERB_VERB) # takes biggest
             self.assertEqual(config.fanout, 42)
             self.assertEqual(config.connect_timeout, 14)
