@@ -1589,6 +1589,19 @@ class YAMLGroupLoaderTest(unittest.TestCase):
         self.assertRaises(GroupResolverConfigError, YAMLGroupLoader, f.name)
 
 
+    def test_list_group(self):
+        f = make_temp_file(dedent("""
+            rednecks:
+                bubba: 
+                    - pickup-1
+                    - pickup-2
+                    - tractor-[1-2]""").encode('ascii'))
+        loader = YAMLGroupLoader(f.name)
+        sources = list(loader)
+        resolver = GroupResolver(sources[0])
+        self.assertEqual(resolver.group_nodes('bubba'),
+                [ 'pickup-1,pickup-2,tractor-[1-2]' ])
+
 class GroupResolverYAMLTest(unittest.TestCase):
 
     def setUp(self):
